@@ -931,13 +931,13 @@ int grapple_client_enumusers(grapple_client client,
 	  pthread_mutex_unlock(&clientdata->connection_mutex);
 	  
 	  //If the user is valid
-	  if (serverid != GRAPPLE_USER_UNKNOWN)
+	  if (tmpname)
 	    {
 	      //Run the callback
 	      (*callback)(serverid,tmpname,0,context);
+	      
+	      free(tmpname);
 	    }
-	  if (tmpname)
-	    free(tmpname);
 	}
       else
 	{
@@ -948,6 +948,8 @@ int grapple_client_enumusers(grapple_client client,
       loopa++;
     }
 
+  free(userarray);
+
   return GRAPPLE_OK;
 }
 
@@ -1391,6 +1393,8 @@ int grapple_client_enumgroup(grapple_client client,
       loopa++;
     }
 
+  free(userarray);
+
   return GRAPPLE_OK;
 }
 
