@@ -20,6 +20,8 @@
     michael@linuxgamepublishing.com
 */
 
+#define _XOPEN_SOURCE 500
+
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -269,6 +271,8 @@ static void process_message_relay_to(internal_client_data *client,
   if (flags & GRAPPLE_CONFIRM)
     c2s_confirm_received(client,from,messageid);
 
+  free(outdata);
+
   return;
 }
 
@@ -457,6 +461,9 @@ static void process_message_failover_on(internal_client_data *client,
     }
 
 
+  //Add this socket to the process list, so we will do the UDP processing on it
+  client->socklist=socket_link(client->socklist,client->failoversock);
+
   //Tell the server to see if it can connect
   c2s_failover_tryme(client);
   return;
@@ -496,6 +503,9 @@ static void process_message_failover_can(internal_client_data *client,
     {
       if (client->failoversock)
 	{
+	  //Remove it from the socket list too
+	  client->socklist=socket_unlink(client->socklist,
+					 client->failoversock);
 	  socket_destroy(client->failoversock);
 	  client->failoversock=NULL;
 	}
@@ -527,6 +537,9 @@ static void process_message_failover_cant(internal_client_data *client,
       //This is telling us we cant failover, close the remote socket
       if (client->failoversock)
 	{
+	  //Remove it from the socket list too
+	  client->socklist=socket_unlink(client->socklist,
+					 client->failoversock);
 	  socket_destroy(client->failoversock);
 	  client->failoversock=NULL;
 	}
@@ -772,6 +785,9 @@ static int client_run_failover(internal_client_data *client)
   grapple_group_container *container,*newcontainer;
   int maxval;
   internal_server_data *serverdata;
+  grapple_connection *newuser;
+  pthread_mutexattr_t attr;
+
 
   //Find the ID of the lowest possible server
   newhost=failover_locate_lowest_id(client->failoverhosts);
@@ -805,6 +821,9 @@ static int client_run_failover(internal_client_data *client)
       grapple_server_session_set(server,client->session);
 
       serverdata=internal_server_get(server);
+
+      //Set the time when the clients ability to reconnect expires
+      serverdata->reconnect_expire=time(NULL)+60;
       
       //Find the highest number of serverid in use, and incriment it by one
       //to the new server id, so we dont get conflicts with the new
@@ -847,14 +866,10 @@ static int client_run_failover(internal_client_data *client)
 	    {
 	      newcontainer=group_container_aquire(container->id);
 
-	      pthread_mutex_lock(&serverdata->group_mutex);
-
 	      //Add this new group member to the group
 	      newgroup->contents=group_container_link(newgroup->contents,
 						      newcontainer);
 
-	      pthread_mutex_unlock(&serverdata->group_mutex);
-
 	      container=container->next;
 	      if (container==groupscan->contents)
 		container=NULL;
@@ -869,6 +884,41 @@ static int client_run_failover(internal_client_data *client)
       //Incriment the server connection ID to be correct
       serverdata->user_serverid=maxval+2;
 
+
+      //Now - for holding user data that may be required later, we add in
+      //a set of dummy users
+
+      //Create the required thread mutex context
+      pthread_mutexattr_init(&attr);
+      pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
+
+      pthread_mutex_lock(&client->connection_mutex);
+      
+      connscan=client->userlist;
+      while (connscan)
+	{
+	  //Create the struct to hold the data
+	  newuser=connection_struct_aquire();
+
+	  //Asign a new server ID
+	  newuser->serverid=connscan->serverid;
+	  newuser->handshook=1;
+
+	  //Create the mutex
+	  pthread_mutex_init(&newuser->message_in_mutex,&attr);
+	  pthread_mutex_init(&newuser->message_out_mutex,&attr);
+	  pthread_mutex_init(&newuser->confirm_mutex,&attr);
+	  
+	  //Link this into the server
+	  serverdata->userlist=connection_link(serverdata->userlist,newuser);
+
+	  connscan=connscan->next;
+	  if (connscan==client->userlist)
+	    connscan=NULL;
+	}
+
+      pthread_mutex_unlock(&client->connection_mutex);
+
       //ONLY NOW start the server
       grapple_server_start(server);
 
@@ -892,12 +942,12 @@ static int client_run_failover(internal_client_data *client)
     {
     case GRAPPLE_PROTOCOL_TCP:
       retry=0;
-      while (retry<400)
+      while (retry<6000)
 	{
 	  newsock=socket_create_inet_tcp_wait(client->address,client->port,1);
 	  if (newsock && socket_connected(newsock))
 	    {
-	      retry=400;
+	      retry=6000;
 	    }
 	  else
 	    {
@@ -1420,8 +1470,8 @@ static void *grapple_client_thread_main(void *voiddata)
   internal_grapple_group *group;
   struct sigaction sa;
 
+  memset(&sa,0,sizeof(struct sigaction));
   sa.sa_handler = SIG_IGN;
-  sa.sa_flags = 0;
   sigaction(SIGPIPE, &sa, 0);
 
   //The client we have started
